@@ -42,13 +42,28 @@ est = function(L, X, rx, conf.level=0.95, adj="lsd", paired=FALSE)
       Pval = 1 - pf(Fval, DFg, rx$DFr)
       DL = sqrt(DFg*qf(conf.level, DFg, rx$DFr))*SE
     } else if (tolower(adj) == "dunnett") {
-      mC = matrix(0.5, nrow=nL, ncol=nL) + diag(0.5, nrow=nL)
+      ColNames = colnames(L)
+      Lf = colSums(L)
+      Groups = ColNames[Lf == 1]
+      Control = ColNames[Lf == -nL]
+      nX = diag(crossprod(X))
+      nControl = nX[Control]
+      nGroups = nX[Groups]
+      vSD = sqrt(nGroups/(nGroups + nControl))
+      mC = outer(vSD, vSD, "*")
+      diag(mC) = 1
+#      mC = matrix(0.5, nrow=nL, ncol=nL) + diag(0.5, nrow=nL) # for balanced data only!
+      if (exists(".Random.seed")) Saved.seed = .Random.seed
+      set.seed(5)
+      D.crit = qmvt(0.5 + conf.level/2, df=rx$DFr, corr=mC)$quantile
       Pval = rep(NA, nL)
       DL = rep(NA, nL)
       for (k in 1:nL) {
+        set.seed(5) # DescTools::DunnettTest forgot to set seed before pmvt
         Pval[k] = 1 - pmvt(lower=rep(-abs(Tval[k]), nL), upper=rep(abs(Tval[k]), nL), df=rx$DFr, corr=mC)
-        DL[k] = qmvt(0.5 + conf.level/2, df=rx$DFr, corr=mC)$quantile*SE[k]
+        DL[k] = D.crit*SE[k]
       }
+      if (exists("Saved.seed")) .Random.seed <<- Saved.seed
     } else {
       stop(paste0("Adjustment method ", adj, " is not supported!"))
     }
